#!/usr/bin/perl
###############################################################################
#                                                                             #
# IPFire.org - A linux based firewall                                         #
# Copyright (C) 2011  IPFire Team  <info@ipfire.org>                          #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
###############################################################################
# New function for forwarding firewall. To make it comfortable to create	  #
# rules, we need "spelling names" for single Hosts. If you have any questions #
# <amarx@ipfire.org>														  #
###############################################################################
use strict;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';
no warnings 'uninitialized';
require '/var/ipfire/general-functions.pl';
require "${General::swroot}/lang.pl";
require "${General::swroot}/header.pl";

my %fwhostsettings=();
my %customnetwork=();
my %customhost=();
my %customgrp=();
my %ccdnet=();
my %ccdhost=();
my %color=();
my %defaultNetworks=();
my %mainsettings=();
my %ownnet=();

my $errormessage;
my $update=0;
my $confignet		= "${General::swroot}/fwhosts/customnetworks";
my $confighost	= "${General::swroot}/fwhosts/customhosts";
my $configgrp 		= "${General::swroot}/fwhosts/customgroups";
my $configccdnet 	= "${General::swroot}/ovpn/ccd.conf";
my $configccdhost	= "${General::swroot}/ovpn/ovpnconfig";

unless (-e $confignet)    { system("touch $confignet"); }
unless (-e $confighost)   { system("touch $confighost"); }
unless (-e $configgrp)    { system("touch $configgrp"); }

&General::readhash("${General::swroot}/main/settings", \%mainsettings);
&General::readhash("/srv/web/ipfire/html/themes/".$mainsettings{'THEME'}."/include/colors.txt", \%color);
&General::readhash("${General::swroot}/ethernet/settings", \%ownnet);
&Header::getcgihash(\%fwhostsettings);

&Header::showhttpheaders();
&Header::openpage($Lang::tr{'fwhost hosts'}, 1, '');
&Header::openbigbox('100%', 'center');

############
# ACTION   #
############
if ($fwhostsettings{'ACTION'} eq 'updatenet' )
{
	&General::readhasharray("$confignet", \%customnetwork);
	foreach my $key (keys %customnetwork)
	{
		if($customnetwork{$key}[0] eq $fwhostsettings{'orgname'})
		{
			$fwhostsettings{'orgname'} 	= $customnetwork{$key}[0];
			$fwhostsettings{'orgip'} 		= $customnetwork{$key}[1];
			$fwhostsettings{'orgsub'} 	= $customnetwork{$key}[2];
			$fwhostsettings{'count'} 		= $customnetwork{$key}[3];
			delete $customnetwork{$key};
			
		}
	}
	&General::writehasharray("$confignet", \%customnetwork);
	$fwhostsettings{'actualize'} = 'on';
	$fwhostsettings{'ACTION'} = 'savenet';
}
###
if ($fwhostsettings{'ACTION'} eq 'savenet' )
{
	my $count=0;
	
	if ($fwhostsettings{'orgname'} eq ''){$fwhostsettings{'orgname'}=$fwhostsettings{'HOSTNAME'};}
	
	#check if all fields are set
	if ($fwhostsettings{'HOSTNAME'} eq '' || $fwhostsettings{'IP'} eq '' || $fwhostsettings{'SUBNET'} eq '')
	{
		$errormessage=$errormessage.$Lang::tr{'fwhost err empty'};
		&addnet;
		&viewtablenet;
	}else{
		#check valid ip 
		if (!&General::validipandmask($fwhostsettings{'IP'}."/".$fwhostsettings{'SUBNET'}))
		{
			$errormessage=$errormessage.$Lang::tr{'fwhost err addr'};
			$fwhostsettings{'BLK_HOST'}	='readonly';
			$fwhostsettings{'NOCHECK'}	='false';
			$fwhostsettings{'error'}='on';
		}
		#check if subnet is sigle host
		if(&General::iporsubtocidr($fwhostsettings{'SUBNET'}) eq '32')
		{
			$errormessage=$errormessage.$Lang::tr{'fwhost err sub32'};
			
		}
		if($fwhostsettings{'error'} ne 'on'){
			#check if we use one of ipfire's networks (green,orange,blue)
			if (($ownnet{'GREEN_NETADDRESS'}  	ne '' && $ownnet{'GREEN_NETADDRESS'} 	ne '0.0.0.0') && &General::IpInSubnet($ownnet{'GREEN_NETADDRESS'},$fwhostsettings{'IP'},&General::iporsubtodec($fwhostsettings{'SUBNET'})))
			{ 
				$errormessage=$errormessage.$Lang::tr{'ccd err green'}."<br>";
				$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}='editnet';}
			}
			if (($ownnet{'ORANGE_NETADDRESS'}	ne '' && $ownnet{'ORANGE_NETADDRESS'} 	ne '0.0.0.0') && &General::IpInSubnet($ownnet{'ORANGE_NETADDRESS'},$fwhostsettings{'IP'},&General::iporsubtodec($fwhostsettings{'SUBNET'})))
			{ 
				$errormessage=$errormessage.$Lang::tr{'ccd err orange'}."<br>";
				$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}='editnet';}
			}
			if (($ownnet{'BLUE_NETADDRESS'} 	ne '' && $ownnet{'BLUE_NETADDRESS'} 	ne '0.0.0.0') && &General::IpInSubnet($ownnet{'BLUE_NETADDRESS'},$fwhostsettings{'IP'},&General::iporsubtodec($fwhostsettings{'SUBNET'})))
			{ 
				$errormessage=$errormessage.$Lang::tr{'ccd err blue'}."<br>";
				$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}='editnet';}
			}
			if (($ownnet{'RED_NETADDRESS'} 	ne '' && $ownnet{'RED_NETADDRESS'} 		ne '0.0.0.0') && &General::IpInSubnet($ownnet{'RED_NETADDRESS'},$fwhostsettings{'IP'},&General::iporsubtodec($fwhostsettings{'SUBNET'})))
			{ 
				$errormessage=$errormessage.$Lang::tr{'ccd err red'}."<br>";
				$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}='editnet';}
			}
		}
		&plausicheck("editnet");
		
		#check if network ip is part of an already used one 
		if(&checksubnet(\%customnetwork))
		{
			$errormessage=$errormessage.$Lang::tr{'fwhost err partofnet'};
			$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
		}				
		
		if($fwhostsettings{'actualize'} eq 'on' && $errormessage)
		{
			$fwhostsettings{'actualize'} = '';
			my $key = &General::findhasharraykey (\%customnetwork);
			foreach my $i (0 .. 3) { $customnetwork{$key}[$i] = "";}
			$customnetwork{$key}[0] = $fwhostsettings{'orgname'} ;
			$customnetwork{$key}[1] = $fwhostsettings{'orgip'} ;
			$customnetwork{$key}[2] = $fwhostsettings{'orgsub'};
			$customnetwork{$key}[3] = $fwhostsettings{'count'};
			&General::writehasharray("$confignet", \%customnetwork);
			undef %customnetwork;
		} 			
					
					
		if (!$errormessage){
			&General::readhasharray("$confignet", \%customnetwork);
			if ($fwhostsettings{'ACTION'} eq 'updatenet'){
				if ($fwhostsettings{'update'} == '0'){
					foreach my $key (keys %customnetwork) {
						if($customnetwork{$key}[0] eq $fwhostsettings{'OLDNAME'}){
							$count=$customnetwork{$key}[3];
							delete $customnetwork{$key};
							last;
						}
					}
				}
			}
									
			my $key = &General::findhasharraykey (\%customnetwork);
			foreach my $i (0 .. 3) { $customnetwork{$key}[$i] = "";}
			$fwhostsettings{'SUBNET'}=&General::iporsubtocidr($fwhostsettings{'SUBNET'});
			$customnetwork{$key}[0] = $fwhostsettings{'HOSTNAME'};
			$customnetwork{$key}[1] = &General::getnetworkip($fwhostsettings{'IP'},$fwhostsettings{'SUBNET'}) ;
			$customnetwork{$key}[2] = &General::iporsubtodec($fwhostsettings{'SUBNET'}) ;
			$customnetwork{$key}[3] = $count;
			&General::writehasharray("$confignet", \%customnetwork);
			&updategrp;
			undef %customnetwork;
			$fwhostsettings{'HOSTNAME'}='';
			$fwhostsettings{'IP'}='';
			$fwhostsettings{'SUBNET'}='';
			&addnet;
			&viewtablenet;
		}else
		{
			&addnet;
			&viewtablenet;
		}
	}
	
}
###
if ($fwhostsettings{'ACTION'} eq 'updatehost')
{
	&General::readhasharray("$confighost", \%customhost);
	foreach my $key (keys %customhost)
	{
		if($customhost{$key}[0] eq $fwhostsettings{'orgname'})
		{
			$fwhostsettings{'orgname'} = $customhost{$key}[0];
			my ($ip,$subnet) = split (/\//,$customhost{$key}[2]);
			$fwhostsettings{'orgip'} = $ip;
			$fwhostsettings{'count'} = $customhost{$key}[3];
			delete $customhost{$key};
		}
	}
	&General::writehasharray("$confighost", \%customhost);
	$fwhostsettings{'actualize'} = 'on';
	$fwhostsettings{'ACTION'} = 'savehost';
}
###
if ($fwhostsettings{'ACTION'} eq 'savehost')
{
	my $count=0;
	if ($fwhostsettings{'orgname'} eq ''){$fwhostsettings{'orgname'}=$fwhostsettings{'HOSTNAME'};}
	
	$fwhostsettings{'SUBNET'}='32';
		
	#check if all fields are set
	if ($fwhostsettings{'HOSTNAME'} eq '' || $fwhostsettings{'IP'} eq '' || $fwhostsettings{'SUBNET'} eq '')
	{
		$errormessage=$errormessage.$Lang::tr{'fwhost err empty'};
		$fwhostsettings{'ACTION'} = 'edithost';
	}else{
	
		if($fwhostsettings{'type'} eq 'ip' && $fwhostsettings{'IP'}=~/^([0-9a-fA-F]{1,2}:){5}[0-9a-fA-F]{1,2}$/)
		{ 	$fwhostsettings{'type'} = 'mac';}
		if($fwhostsettings{'type'} eq 'mac' && $fwhostsettings{'IP'}=~/^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/)
		{ 	$fwhostsettings{'type'} = 'ip';}
		if($fwhostsettings{'type'} eq 'mac' )
		{
			if ($fwhostsettings{'IP'}!~/^([0-9a-fA-F]{1,2}:){5}[0-9a-fA-F]{1,2}$/ )
			{
				$errormessage=$Lang::tr{'fwhost err mac'};
			}
		}
		
		#check for subnet
		if (rindex($fwhostsettings{'IP'},'/') eq '-1'){
			if($fwhostsettings{'type'} eq 'ip' && !&General::validipandmask($fwhostsettings{'IP'}."/32"))
				{
					$errormessage=$errormessage.$Lang::tr{'fwhost err ip'};
				}
			
		}else{
			
			$fwhostsettings{'NOCHECK'}="false";
			$errormessage=$errormessage.$Lang::tr{'fwhost err ipwithsub'};
			
		}
		#check if net or broadcast
		my @tmp= split (/\./,$fwhostsettings{'IP'});
		if (($tmp[3] eq "0") || ($tmp[3] eq "255"))
		{
			$errormessage=$Lang::tr{'fwhost err hostip'};
		}
			
		&plausicheck("edithost");
		
		
		
		if($fwhostsettings{'actualize'} eq 'on' && $errormessage)
		{
			$fwhostsettings{'actualize'} = '';
			my $key = &General::findhasharraykey (\%customhost);
			foreach my $i (0 .. 3) { $customhost{$key}[$i] = "";}
			$customhost{$key}[0] = $fwhostsettings{'orgname'} ;
			$customhost{$key}[1] = $fwhostsettings{'type'} ;
			$customhost{$key}[2] = $fwhostsettings{'orgip'}."/".&General::iporsubtodec($fwhostsettings{'SUBNET'});
			$customhost{$key}[3] = $fwhostsettings{'count'};
			&General::writehasharray("$confighost", \%customhost);
			#&updategrp;
			undef %customhost;
		} 
		
		if (!$errormessage){
			my $key = &General::findhasharraykey (\%customhost);
			foreach my $i (0 .. 3) { $customhost{$key}[$i] = "";}
			$customhost{$key}[0] = $fwhostsettings{'HOSTNAME'} ;
			$customhost{$key}[1] = $fwhostsettings{'type'} ;
			$customhost{$key}[2] = $fwhostsettings{'IP'}."/".&General::iporsubtodec($fwhostsettings{'SUBNET'});
			$customhost{$key}[3] = $count;
			&General::writehasharray("$confighost", \%customhost);
			&updategrp;
			undef %customhost;
			$fwhostsettings{'HOSTNAME'}='';
			$fwhostsettings{'IP'}='';
			$fwhostsettings{'type'}='';
			&addhost;
			&viewtablehost;
		}else{
			&addhost;
			&viewtablehost;
		}
	
	}

}
###
if ($fwhostsettings{'ACTION'} eq 'savegrp')
{
	my $grp;
	my $rem=$fwhostsettings{'remark'};
	my $count;
	my $type;
	my @target;
	my @newgrp;
	&General::readhasharray("$configgrp", \%customgrp);
	$grp=$fwhostsettings{'grp_name'};
	
	if (!&General::validhostname($grp)){$errormessage=$errormessage.$Lang::tr{'fwhost err name'};}
	
	###check standard networks
	if ($fwhostsettings{'grp2'} eq 'std_net'){
		if ($fwhostsettings{'DEFAULT_SRC_ADR'} =~ /OpenVPN/i){
			my %ovpn=();
			&General::readhash("${General::swroot}/ovpn/settings",\%ovpn);
			@target="OpenVPN,".$ovpn{'DOVPN_SUBNET'};
		}
		if ($fwhostsettings{'DEFAULT_SRC_ADR'} =~ /Gr/i){
			my %hash=();
			&General::readhash("${General::swroot}/ethernet/settings",\%hash);
			@target=$Lang::tr{'fwhost green'}.",".$hash{'GREEN_NETADDRESS'}."/".$hash{'GREEN_NETMASK'};
		}
		if ($fwhostsettings{'DEFAULT_SRC_ADR'} =~ /Bl/i){
			my %hash=();
			&General::readhash("${General::swroot}/ethernet/settings",\%hash);
			@target=$Lang::tr{'fwhost blue'}.",".$hash{'BLUE_NETADDRESS'}."/".$hash{'BLUE_NETMASK'};
		}
		if ($fwhostsettings{'DEFAULT_SRC_ADR'} =~ /Ora/i){
			my %hash=();
			&General::readhash("${General::swroot}/ethernet/settings",\%hash);
			@target=$Lang::tr{'fwhost orange'}.",".$hash{'ORANGE_NETADDRESS'}."/".$hash{'ORANGE_NETMASK'};
		}
		if ($fwhostsettings{'DEFAULT_SRC_ADR'} =~ /Any/i || $fwhostsettings{'DEFAULT_SRC_ADR'} =~ /Alle/i){@target=$Lang::tr{'fwhost any'}.",0.0.0.0/0.0.0.0";}
		$type='Standard Network';	
	}
	##check custom networks
	if ($fwhostsettings{'grp2'} eq 'cust_net' && $fwhostsettings{'CUST_SRC_NET'} ne ''){
		&General::readhasharray("$confignet", \%customnetwork);
		foreach my $key (keys %customnetwork) {
			if($customnetwork{$key}[0] eq $fwhostsettings{'CUST_SRC_NET'}){
				@target=$customnetwork{$key}[0].",".$customnetwork{$key}[1]."/".$customnetwork{$key}[2];
				$customnetwork{$key}[3] = $customnetwork{$key}[3]+1;
				last;
			}
		}
		#Increase count number in added network
		&General::writehasharray("$confignet", \%customnetwork);
		$type='Custom Network';
	}elsif($fwhostsettings{'grp2'} eq 'cust_net' && $fwhostsettings{'CUST_SRC_NET'} eq ''){
		$errormessage=$Lang::tr{'fwhost err groupempty'}."<br>";
	}
	#check custom addresses
	if ($fwhostsettings{'grp2'} eq 'cust_host' && $fwhostsettings{'CUST_SRC_HOST'} ne ''){
		&General::readhasharray("$confighost", \%customhost);
		foreach my $key (keys %customhost) {
			if ($customhost{$key}[0] eq $fwhostsettings{'CUST_SRC_HOST'}){
				@target=$customhost{$key}[0].",".$customhost{$key}[2];
				$customhost{$key}[3]=$customhost{$key}[3]+1;
				last;
			}
		}
		#Increase count number in added host
		&General::writehasharray("$confighost", \%customhost);
		$type='Custom Host';
	}elsif($fwhostsettings{'grp2'} eq 'cust_host' && $fwhostsettings{'CUST_SRC_HOST'} eq ''){
		$errormessage=$Lang::tr{'fwhost err groupempty'}."<br>";
	}
		
	#get address from  ovpn ccd static net
	if ($fwhostsettings{'grp2'} eq 'ovpn_net' && $fwhostsettings{'OVPN_CCD_NET'} ne ''){
		&General::readhasharray("$configccdnet", \%ccdnet);
		foreach my $key (keys %ccdnet) {
			if ($ccdnet{$key}[0] eq $fwhostsettings{'OVPN_CCD_NET'}){
				my ($a,$b) = split (/\//,$ccdnet{$key}[1]);
				$b=&General::iporsubtodec($b);
				@target=$ccdnet{$key}[0].",$a/$b";
				last;
			}
		}
		$type='OpenVPN static network';
	}elsif($fwhostsettings{'grp2'} eq 'ovpn_net' && $fwhostsettings{'OVPN_CCD_NET'} eq ''){
		$errormessage=$Lang::tr{'fwhost err groupempty'};
	}
	#get address from ovpn ccd static host
	if ($fwhostsettings{'grp2'} eq 'ovpn_host' && $fwhostsettings{'OVPN_CCD_HOST'} ne ''){
		&General::readhasharray("$configccdhost", \%ccdhost);
			foreach my $key (keys %ccdhost) {
				if($ccdhost{$key}[1] eq $fwhostsettings{'OVPN_CCD_HOST'}){
					my ($a,$b) = split (/\//,$ccdhost{$key}[33]);
					$b=&General::iporsubtodec($b);
					@target=$ccdhost{$key}[1].",$a/$b";
					last;
				}
			}
			$type='OpenVPN static host';
	}elsif ($fwhostsettings{'grp2'} eq 'ovpn_host' && $fwhostsettings{'OVPN_CCD_HOST'} eq ''){
		$errormessage=$Lang::tr{'fwhost err groupempty'};
	}
	#get address from ovpn ccd Net-2-Net
	if ($fwhostsettings{'grp2'} eq 'ovpn_n2n' && $fwhostsettings{'OVPN_N2N'} ne ''){
		&General::readhasharray("$configccdhost", \%ccdhost);
		foreach my $key (keys %ccdhost) {
			if($ccdhost{$key}[1] eq $fwhostsettings{'OVPN_N2N'}){
				my ($a,$b) = split ("/",$ccdhost{$key}[11]);
				$b=&General::iporsubtodec($b);
				@target=$ccdhost{$key}[1].",$a/$b";
				last;
			}
		}
		$type='OpenVPN N-2-N';
	}elsif ($fwhostsettings{'grp2'} eq 'ovpn_n2n' && $fwhostsettings{'OVPN_N2N'} eq ''){
		$errormessage=$Lang::tr{'fwhost err groupempty'};
	}
	
	
	#check if host/net exists in grp
	my $test="$grp,$fwhostsettings{'oldremark'},@target";
	foreach my $key (keys %customgrp) {
		my $test1="$customgrp{$key}[0],$customgrp{$key}[1],$customgrp{$key}[2],$customgrp{$key}[3]";
		if ($test1 eq $test){
			$errormessage=$Lang::tr{'fwhost err isingrp'};
			$fwhostsettings{'update'} = 'on';
		}
	}
		
	if (!$errormessage){
		#check if remark has also changed
		if ($fwhostsettings{'remark'} ne $fwhostsettings{'oldremark'})
		{
			foreach my $key (keys %customgrp)
			{
				if($customgrp{$key}[0] eq $grp && $customgrp{$key}[1] eq $fwhostsettings{'oldremark'})
				{
					$customgrp{$key}[1]='';
					$customgrp{$key}[1]=$rem;
				}	
			}
		}
		#get count used
		foreach my $key (keys %customgrp)
		{
			if($customgrp{$key}[0] eq $grp)
			{
				$count=$customgrp{$key}[5];
				last;
			}
		}
		if ($count eq '' ){$count='0';}
		
		#create array with new lines
		foreach my $line (@target){
			push (@newgrp,"$grp,$rem,$line");
		}
		#append new entries
		my $key = &General::findhasharraykey (\%customgrp);
		foreach my $line (@newgrp){
			foreach my $i (0 .. 4) { $customgrp{$key}[$i] = "";}
			my ($a,$b,$c,$d) = split (",",$line);
			$customgrp{$key}[0] = $a;
			$customgrp{$key}[1] = $b;
			$customgrp{$key}[2] = $c;
			$customgrp{$key}[3] = $d;
			$customgrp{$key}[4] = $type;
			$customgrp{$key}[5] = $count;
		}
		&General::writehasharray("$configgrp", \%customgrp);
		
		$fwhostsettings{'update'}='on';
		
	}
		if ($fwhostsettings{'remark'} ne $fwhostsettings{'oldremark'} && $errormessage)
		{
			foreach my $key (keys %customgrp)
			{
				if($customgrp{$key}[0] eq $grp && $customgrp{$key}[1] eq $fwhostsettings{'oldremark'})
				{
					$customgrp{$key}[1]='';
					$customgrp{$key}[1]=$rem;
				}	
			}
			&General::writehasharray("$configgrp", \%customgrp);
			$errormessage=$Lang::tr{'fwhost changeremark'};
			$fwhostsettings{'update'}='on';
		}
		#check if ruleupdate is needed
		if($count > 0 )
		{
			&rules;
		}
		&addgrp;
		&viewtablegrp;
	
}
###
if ($fwhostsettings{'ACTION'} eq 'editnet')
{
	print" OLDNAME= $fwhostsettings{'orgname'}";
	&General::readhasharray("$confignet", \%customnetwork);
	foreach my $key (keys %customnetwork) {
		if($customnetwork{$key}[0] eq $fwhostsettings{'orgname'}){
			$fwhostsettings{'HOSTNAME'} 	= $customnetwork{$key}[0];
			$fwhostsettings{'IP'} 		= $customnetwork{$key}[1];
			$fwhostsettings{'SUBNET'}		= $customnetwork{$key}[2];
			
			last;
		}
	}
	if ($fwhostsettings{'update'} ne 'on')
	{
		&addnet;
		&viewtablenet;
	}
}
###
if ($fwhostsettings{'ACTION'} eq 'edithost')
{
	&General::readhasharray("$confighost", \%customhost);
	foreach my $key (keys %customhost) {
		if($customhost{$key}[0] eq $fwhostsettings{'orgname'}){
			my ($ip,$subnet) = split (/\//, $customhost{$key}[2]);
			$fwhostsettings{'HOSTNAME'} = $customhost{$key}[0];
			$fwhostsettings{'IP'} 		= $ip;
			last;
		}
	}
	if ($fwhostsettings{'update'} ne 'on')
	{
		&addhost;
		&viewtablehost;
	}
}
###
if ($fwhostsettings{'ACTION'} eq 'editgrp')
{
	$fwhostsettings{'update'}='on';
	&addgrp;
	&viewtablegrp;
}
###
if ($fwhostsettings{'ACTION'} eq 'resetnet')
{
	$fwhostsettings{'HOSTNAME'} ="";
	$fwhostsettings{'IP'} 		="";
	$fwhostsettings{'SUBNET'}	="";
	&showmenu;
}
###
if ($fwhostsettings{'ACTION'} eq 'resethost')
{
	$fwhostsettings{'HOSTNAME'} ="";
	$fwhostsettings{'IP'} 		="";
	$fwhostsettings{'type'} 	="";
	&showmenu;
}
###
if ($fwhostsettings{'ACTION'} eq $Lang::tr{'fwhost reset'})
{
	
	&showmenu;
	
}
###
if ($fwhostsettings{'ACTION'} eq $Lang::tr{'fwhost newnet'})
{
	&addnet;
	&viewtablenet;
}
###
if ($fwhostsettings{'ACTION'} eq $Lang::tr{'fwhost newhost'})
{
	&addhost;
	&viewtablehost;
}
###
if ($fwhostsettings{'ACTION'} eq $Lang::tr{'fwhost newgrp'})
{
	&addgrp;
	&viewtablegrp;
}
###
if ($fwhostsettings{'ACTION'} eq 'delnet')
{
	&General::readhasharray("$confignet", \%customnetwork);
	foreach my $key (keys %customnetwork) {
		if($fwhostsettings{'key'} eq $customnetwork{$key}[0]){
			&deletefromgrp($customnetwork{$key}[0],$confignet);
			delete $customnetwork{$key};
			&General::writehasharray("$confignet", \%customnetwork);
			last;
		}
	}
	&addnet;
	&viewtablenet;
}
###
if ($fwhostsettings{'ACTION'} eq 'delhost')
{
	&General::readhasharray("$confighost", \%customhost);
	foreach my $key (keys %customhost) {
		if($fwhostsettings{'key'} eq $customhost{$key}[0]){
			&deletefromgrp($customhost{$key}[0],$configgrp);
			delete $customhost{$key};
			&General::writehasharray("$confighost", \%customhost);
			last;
		}
	}
	&addhost;
	&viewtablehost;
	
}
###
if ($fwhostsettings{'ACTION'} eq 'deletegrphost')
{
	&General::readhasharray("$configgrp", \%customgrp);
	foreach my $key (keys %customgrp){
		if($customgrp{$key}[0].",".$customgrp{$key}[1].",".$customgrp{$key}[2].",".$customgrp{$key}[3].",".$customgrp{$key}[4] eq $fwhostsettings{'delhost'}){
			#decrease count from source host/net
			if ($customgrp{$key}[4] eq 'Custom Network'){
				&General::readhasharray("$confignet", \%customnetwork);
				foreach my $key1 (keys %customnetwork){
						if ($customnetwork{$key1}[0] eq $customgrp{$key}[2]){
						$customnetwork{$key1}[3] = $customnetwork{$key1}[3]-1;
						last;
					}
				}
				&General::writehasharray("$confignet", \%customnetwork);
			}
			if ($customgrp{$key}[4] eq 'Custom Host'){
				&General::readhasharray("$confighost", \%customhost);
				foreach my $key1 (keys %customhost){
					if ($customhost{$key1}[0] eq $customgrp{$key}[2]){
						$customhost{$key1}[3] = $customhost{$key1}[3]-1;
						last;
					}
				}
				&General::writehasharray("$confighost", \%customhost);
			}
			delete $customgrp{$key};
		}
	}
	&General::writehasharray("$configgrp", \%customgrp);
	&rules;
	&addgrp;
	&viewtablegrp;
}
###
if ($fwhostsettings{'ACTION'} eq 'delgrp')
{
	&General::readhasharray("$configgrp", \%customgrp);
	&decrease($fwhostsettings{'grp_name'});
	foreach my $key (sort keys %customgrp)
	{
		if($customgrp{$key}[0] eq $fwhostsettings{'grp_name'})
		{
			delete $customgrp{$key};
		}
	}
	&General::writehasharray("$configgrp", \%customgrp);
	$fwhostsettings{'grp_name'}='';
	&addgrp;
	&viewtablegrp;
}
#########
# View  #
#########

if($fwhostsettings{'ACTION'} eq ''){
	&showmenu;
}
##############
# Functions  #
##############
sub showmenu
{
	
	&Header::openbox('100%', 'left',$Lang::tr{'fwhost menu'});
	print<<END;
	<table border='0' width='100%'><form method='post'>
	<tr><td><input type='submit' name='ACTION' value='$Lang::tr{'fwhost newnet'}' /><input type='submit' name='ACTION' value='$Lang::tr{'fwhost newhost'}' /><input type='submit' name='ACTION' value='$Lang::tr{'fwhost newgrp'}' /></td></tr>
	<tr><td colspan='6'><hr></hr></td></tr></table></form>
END
	
	&Header::closebox();
	&viewtablenet;
	&viewtablehost;
	&viewtablegrp;
}
sub showaddnet
{
	&addnet;
	&viewtablenet;
}
sub addnet
{
	&error;
	&Header::openbox('100%', 'left', $Lang::tr{'fwhost addnet'});
	$fwhostsettings{'orgname'}=$fwhostsettings{'HOSTNAME'};
	print<<END;
	<table border='0' width='100%'><form method='post' style='display:inline'  >
	<tr><td><b>$Lang::tr{'name'}:</td><td><input type='TEXT' name='HOSTNAME' value='$fwhostsettings{'HOSTNAME'}' $fwhostsettings{'BLK_HOST'}></td><td><b>$Lang::tr{'fwhost ipadr'}</td><td><input type='TEXT' name='IP' value='$fwhostsettings{'IP'}' $fwhostsettings{'BLK_IP'} size='14'></td><td align='right'><b>$Lang::tr{'netmask'}:</td><td align='right'><input type='TEXT' name='SUBNET' value='$fwhostsettings{'SUBNET'}' $fwhostsettings{'BLK_IP'} size='14'></td></tr>
	<tr><td colspan='6'><hr></hr></td></tr><tr>
END
	if ($fwhostsettings{'ACTION'} eq 'editnet')
	{
		print "<td colspan='6' align='right' ><input type='submit' value='$Lang::tr{'update'}'><input type='hidden' name='ACTION' value='updatenet'><input type='hidden' name='orgname' value='$fwhostsettings{'orgname'}' ><input type='hidden' name='update' value='on'>";
	}else{
		print "<td colspan='6' align='right'><input type='submit' value='$Lang::tr{'save'}' /><input type='hidden' name='ACTION' value='savenet'>";
	}	
	print "</form><form method='post' style='display:inline'><input type='submit' value='$Lang::tr{'fwhost back'}' ><input type='hidden' name='ACTION' value='resetnet'></td></tr></table></form>";
	&Header::closebox();
}
sub addhost
{
	&error;
	&Header::openbox('100%', 'left', $Lang::tr{'fwhost addhost'});
	$fwhostsettings{'orgname'}=$fwhostsettings{'HOSTNAME'};
	print<<END;
	<table border='0' width='100%'><form method='post' style='display:inline'>
	<tr><td><b>$Lang::tr{'name'}:</td><td width='30%'><input type='TEXT' name='HOSTNAME' value='$fwhostsettings{'HOSTNAME'}' $fwhostsettings{'BLK_HOST'} ></td><td><select name='type'>
END
	if ($fwhostsettings{'type'} eq 'ip'){print "<option value='ip' selected >IP</option>";}else{print "<option value='ip' >IP</option>";}
	if ($fwhostsettings{'type'} eq 'mac'){print "<option value='mac' selected >MAC</option>";}else{print "<option value='mac' >MAC</option>";}
	print<<END;
	</option></select></td><td><b>IP/MAC:</td><td><input type='TEXT' name='IP' value='$fwhostsettings{'IP'}' $fwhostsettings{'BLK_IP'} ></td></tr>
	<tr><td colspan='7'><br><br><b>$Lang::tr{'fwhost attention'}</b><br>$Lang::tr{'fwhost macwarn'}</td></tr>
	<tr><td colspan='7'><hr></hr></td></tr>
END

	if ($fwhostsettings{'ACTION'} eq 'edithost')
	{
		
		print "	<td colspan='6' align='right'><input type='submit' value='$Lang::tr{'update'}' /><input type='hidden' name='ACTION' value='updatehost'><input type='hidden' name='orgname' value='$fwhostsettings{'orgname'}' ><input type='hidden' name='update' value='on'></form>";
	}else{
		print "	<td colspan='6' align='right'><input type='submit' name='savehost' value='$Lang::tr{'save'}' /><input type='hidden' name='ACTION' value='savehost' />";
	}	
	print "	</form><form method='post' style='display:inline'><input type='submit' value='$Lang::tr{'fwhost back'}'><input type='hidden' name='ACTION' value='resethost'></td></tr></table></form>";
	&Header::closebox();
}
sub addgrp
{
	&error;
	&Header::openbox('100%', 'left', $Lang::tr{'fwhost addgrp'});
	&setup_default_networks(\%defaultNetworks);
	my %checked=();
	$checked{'check1'}{'off'} = '';
	$checked{'check1'}{'on'} = '';
	$checked{'grp2'}{$fwhostsettings{'grp2'}} = 'CHECKED';
	$fwhostsettings{'oldremark'}=$fwhostsettings{'remark'};
	
	
	print"WERT: $fwhostsettings{grp_name}";
	    
		if ($fwhostsettings{'update'} eq ''){   
			print<<END;
			<table width='100%' border='0'><form method='post'>
			<tr><td nowrap='nowrap' width='16%'>$Lang::tr{'fwhost addgrpname'}</td><td><input type='TEXT' name='grp_name' value='$fwhostsettings{'grp_name'}' size='24'></td><td align='right'>$Lang::tr{'remark'}:</td><td align='right'><input type='TEXT' name='remark' size='30' value='$fwhostsettings{'remark'}'></tr>
			<tr><td colspan='5'><hr></td></tr></table>
END
		}else{
			print<<END;
			<table width='100%' border='0'><form method='post'>
			<tr><td nowrap='nowrap' width='16%'>$Lang::tr{'fwhost addgrpname'}</td><td><input type='TEXT' name='grp_name' size='24' value='$fwhostsettings{'grp_name'}' readonly ></td><td>$Lang::tr{'remark'}:</td><td><input type='TEXT' name='remark' size='30' value='$fwhostsettings{'remark'}'></tr>
			<tr><td colspan='5'><hr></td></tr></table>
END
	
		}
		print<<END;
		<table width='100%' border='0'><tr><td width='1%'><input type='radio' name='grp2' value='std_net'  checked></td><td nowrap='nowrap' width='16%'>$Lang::tr{'fwhost stdnet'}</td><td><select name='DEFAULT_SRC_ADR' style='min-width:185px;'>
		
END
		foreach my $network (sort keys %defaultNetworks)
		{
			next if($defaultNetworks{$network}{'LOCATION'} eq "IPCOP");
			print "<option value='$network'";
			print " selected='selected'" if ($fwhostsettings{'DEFAULT_SRC_ADR'} eq $network);
			print ">$network</option>";
		}

		print<<END;
		</select></td><td width='1%'><input type='radio' name='grp2' value='ovpn_net'  $checked{'grp2'}{'ovpn_net'}></td><td nowrap='nowrap' width='16%'>$Lang::tr{'fwhost ccdnet'}</td><td nowrap='nowrap' width='1%'><select name='OVPN_CCD_NET' style='min-width:185px;'>
END
		&General::readhasharray("$configccdnet", \%ccdnet);
		foreach my $key (sort keys %ccdnet)
		{
			print"<option value='$ccdnet{$key}[0]'>$ccdnet{$key}[0]</option>";
		}
		
		print<<END;
		</select></td></tr>
		<tr><td><input type='radio' name='grp2' value='cust_net' $checked{'grp2'}{'cust_net'}></td><td>$Lang::tr{'fwhost cust net'}</td><td><select name='CUST_SRC_NET' style='min-width:185px;'>
END
		&General::readhasharray("$confignet", \%customnetwork);
		foreach my $key (keys %customnetwork) {
			print"<option>$customnetwork{$key}[0]</option>";
		}
		
		print<<END;
		</select></td><td width='1%'><input type='radio' name='grp2' value='ovpn_host' $checked{'grp2'}{'ovpn_host'}></td><td nowrap='nowrap' width='16%'>$Lang::tr{'fwhost ccdhost'}</td><td nowrap='nowrap' width='1%'><select name='OVPN_CCD_HOST' style='min-width:185px;'>
END
		&General::readhasharray("$configccdhost", \%ccdhost);
		foreach my $key (sort keys %ccdhost)
		{
			if ($ccdhost{$key}[33] ne ''){
				print"<option value='$ccdhost{$key}[1]'>$ccdhost{$key}[1]</option>";
			}
		}
		
		print<<END;
		</select></td></tr>
		<tr><td valign='top'><input type='radio' name='grp2' value='cust_host' $checked{'grp2'}{'cust_host'}></td><td valign='top'>$Lang::tr{'fwhost cust addr'}</td><td><select name='CUST_SRC_HOST' style='min-width:185px;'>
END
		&General::readhasharray("$confighost", \%customhost);
		foreach my $key (keys %customhost) {
			print"<option>$customhost{$key}[0]</option>";
		}
		print<<END;
		</select></td><td width='1%'><input type='radio' name='grp2' value='ovpn_n2n' $checked{'grp2'}{'ovpn_n2n'}></td><td valign='top'>$Lang::tr{'fwhost ovpn_n2n'}</td><td colspan='3'><select name='OVPN_N2N' style='min-width:185px;'>
END
		&General::readhasharray("$configccdhost", \%ccdhost);
		foreach my $key (keys %ccdhost) {
			if($ccdhost{$key}[3] eq 'net'){
				print"<option>$ccdhost{$key}[1]</option>";
			}
		}
		print<<END;
		</select></td></tr>
		</table>
		<br><br><br>
		<b>$Lang::tr{'fwhost attention'}:</b><br>
		$Lang::tr{'fwhost macwarn'}<br><hr>
		<table border='0' width='100%'>
		<tr><td align='right'><input type='submit' value='$Lang::tr{'add'}' style='min-width:100px;' /><input type='hidden' name='oldremark' value='$fwhostsettings{'oldremark'}'><input type='hidden' name='ACTION' value='savegrp' ></form><form method='post' style='display:inline'><input type='submit' value='$Lang::tr{'fwhost back'}' style='min-width:100px;'><input type='hidden' name='ACTION' value'reset'></td></td>
		</table></form>
END
	
	&Header::closebox();
}
sub viewtablenet
{
	if(! -z $confignet){
		&Header::openbox('100%', 'left', $Lang::tr{'fwhost cust net'});
		&General::readhasharray("$confignet", \%customnetwork);
		if (!keys %customnetwork) 
		{ 
			print "<center><b>$Lang::tr{'fwhost empty'}</b>"; 
		}else{
			print<<END;
			<table border='0' width='100%'>
			<tr><td align='center'><b>$Lang::tr{'name'}</td><td align='center'><b>$Lang::tr{'ip address'}</td><td align='center'><b>$Lang::tr{'netmask'}</td><td align='center'><b>$Lang::tr{'used'}</td><td></td><td width='3%'></td></tr>
END
		}
		my $count=0;
		foreach my $key (sort { uc($customnetwork{$a}[0]) cmp uc($customnetwork{$b}[0]) } keys %customnetwork) {
			if ($fwhostsettings{'ACTION'} eq 'editnet' && $fwhostsettings{'HOSTNAME'} eq $customnetwork{$key}[0]) {
				print" <tr bgcolor='#ff3333'>";
			}elsif ($count % 2)
			{ 
				print" <tr bgcolor='$color{'color22'}'>";
			}else
			{
				print" <tr bgcolor='$color{'color20'}'>";
			}
			print"<td width='40%'><form method='post'>$customnetwork{$key}[0]</td><td width=25%'>$customnetwork{$key}[1]</td><td width='25%'>$customnetwork{$key}[2]</td><td align='center'>$customnetwork{$key}[3] x</td>";
			print"<td width='1%'><input type='image' src='/images/edit.gif' align='middle' alt=$Lang::tr{'edit'} title=$Lang::tr{'edit'} /><input type='hidden' name='ACTION' value='editnet'><input type='hidden' name='orgname' value='$customnetwork{$key}[0]' /><input type='hidden' name='key' value='$customnetwork{$key}[0]' /></td></form>";
						
			if($customnetwork{$key}[3] == '0')
			{
				print"<td width='1%'><form method='post'><input type='image' src='/images/delete.gif' align='middle' alt=$Lang::tr{'delete'} title=$Lang::tr{'delete'} /><input type='hidden' name='ACTION' value='delnet' /><input type='hidden' name='key' value='$customnetwork{$key}[0]' /></td></form></tr>";
			}else{
				print"<td width='1%'></td></form></tr>";
			}
			$count++;
		}
		print"</table>";
		&Header::closebox();
}

}
sub viewtablehost
{
	if (! -z $confighost){
		&Header::openbox('100%', 'left', $Lang::tr{'fwhost cust addr'});
		&General::readhasharray("$confighost", \%customhost);
		if (!keys %customhost) 
		{ 
			print "<center><b>$Lang::tr{'fwhost empty'}</b>"; 
		}else{
		print<<END;
		<table border='0' width='100%'>
		<tr><td align='center'><b>$Lang::tr{'name'}</td><td align='center'><b>$Lang::tr{'ip address'}</td><td align='center'><b>$Lang::tr{'used'}</td><td></td><td width='3%'></td></tr>
END
	}
		my $count=0;
		foreach my $key (sort { uc($customhost{$a}[0]) cmp uc($customhost{$b}[0]) } keys %customhost) {
			if ( $fwhostsettings{'ACTION'} eq 'edithost' && $fwhostsettings{'HOSTNAME'} eq $customhost{$key}[0]) {
				print" <tr bgcolor='#ff3333'>";
			}elsif ($count % 2){ print" <tr bgcolor='$color{'color22'}'>";}
			else{            print" <tr bgcolor='$color{'color20'}'>";}
			print"<td width='40%'><form method='post'>$customhost{$key}[0]</td><td width='50%'>$customhost{$key}[2]</td><td align='center'>$customhost{$key}[3] x</td>";
			print"<td width='1%'><input type='image' src='/images/edit.gif' align='middle' alt=$Lang::tr{'edit'} title=$Lang::tr{'edit'} /><input type='hidden' name='ACTION' value='edithost' /><input type='hidden' name='orgname' value='$customhost{$key}[0]' /><input type='hidden' name='type' value='$customhost{$key}[1]' /><input type='hidden' name='key' value='$customhost{$key}[0]' /></td></form>";
			if($customhost{$key}[3] == '0')
			{
				print"<td width='1%'><form method='post'><input type='image' src='/images/delete.gif' align='middle' alt=$Lang::tr{'delete'} title=$Lang::tr{'delete'} /><input type='hidden' name='ACTION' value='delhost' /><input type='hidden' name='key' value='$customhost{$key}[0]' /></td></form></tr>";
			}else{
				print"<td width='1%'></td></tr>";
			}
			$count++;
		}
		print"</table>";
		&Header::closebox();
	}
}
sub viewtablegrp
{
	if(! -z "$configgrp"){
	&Header::openbox('100%', 'left', $Lang::tr{'fwhost cust grp'});
	&General::readhasharray("$configgrp", \%customgrp);
	my @grp=();
	my $helper='';
	my $count=0;
	my $grpname;
	my $remark;
	my $number=keys %customgrp;
	if (!keys %customgrp) 
	{ 
		print "<center><b>$Lang::tr{'fwhost empty'}</b>"; 
	}else{
		foreach my $key (sort { uc($customgrp{$a}[0]) cmp uc($customgrp{$b}[0]) } keys %customgrp){
			$count++;
			if ($helper ne $customgrp{$key}[0]){
				$grpname=$customgrp{$key}[0];
				$remark=$customgrp{$key}[1];
				if($count >=2){print"</table>";}
				print"<br><b><u>$grpname</u></b> &nbsp &nbsp <b>$Lang::tr{'remark'}:</b>&nbsp $remark &nbsp <b>$Lang::tr{'used'}:</b> $customgrp{$key}[5] x";
				if($customgrp{$key}[4] == '0')
				{
					print"<form method='post' style='display:inline'><input type='image' src='/images/delete.gif' alt=$Lang::tr{'delete'} title=$Lang::tr{'delete'} align='right' /><input type='hidden' name='grp_name' value='$grpname' ><input type='hidden' name='ACTION' value='delgrp'></form>";
				}
				print"<form method='post' style='display:inline'><input type='image' src='/images/edit.gif' alt=$Lang::tr{'edit'} title=$Lang::tr{'edit'} align='right' /><input type='hidden' name='grp_name' value='$grpname' ><input type='hidden' name='remark' value='$remark' ><input type='hidden' name='ACTION' value='editgrp'></form>";
				print"<table width='100%' style='border: 1px solid  #000000;' rules='none' ><tr><td align='center'><b>Name</b></td><td align='center'><b>$Lang::tr{'ip address'}</b></td><td align='center' width='25%'><b>$Lang::tr{'fwhost type'}</td></tr>";
			}
			if ( ($fwhostsettings{'ACTION'} eq 'editgrp' || $fwhostsettings{'update'} ne '') && $fwhostsettings{'grp_name'} eq $customgrp{$key}[0]) {
				print" <tr bgcolor='#ff3333'>";
				}elsif ($count %2 == 0){print"<tr bgcolor='$color{'color22'}'>";}else{print"<tr bgcolor='$color{'color20'}'>";}
			print "<td width='39%'>$customgrp{$key}[2]</td><td>$customgrp{$key}[3]</td><td>$customgrp{$key}[4]</td><td width='1%'><form method='post'>";
			if ($number gt '1'){
				print"<input type='image' src='/images/delete.gif' align='middle' alt=$Lang::tr{'delete'} title=$Lang::tr{'delete'} />";
			}
			print"<input type='hidden' name='ACTION' value='deletegrphost'><input type='hidden' name='delhost' value='$grpname,$remark,$customgrp{$key}[2],$customgrp{$key}[3],$customgrp{$key}[4]'></form></td></tr>";
			
			$helper=$customgrp{$key}[0];
		}
		print"</table>";
		
	}
	&Header::closebox();
}

}
sub checkhash
{
	my %hash=%{(shift)};
	foreach my $key (keys %hash) {
		if($hash{$key}[0] eq $fwhostsettings{'HOSTNAME'}){
			return 0;
		}
	}
	return 1;
	
}
sub checkip
{
	
	my %hash=%{(shift)};
	my $a=shift;
	foreach my $key (keys %hash) {
		if($hash{$key}[$a] eq $fwhostsettings{'IP'}."/".&General::iporsubtodec($fwhostsettings{'SUBNET'})){
			return 0;
		}
	}
	return 1;
}
sub checksubnet
{
	
	my %hash=%{(shift)};
	&General::readhasharray("$confignet", \%hash);
	foreach my $key (keys %hash) {
		if(&General::IpInSubnet($fwhostsettings{'IP'},$hash{$key}[1],$hash{$key}[2]))
		{
			return 1;
		}
	}
	return 0;
}
sub error
{
	if ($errormessage) {
		&Header::openbox('100%', 'left', $Lang::tr{'error messages'});
		print "<class name='base'>$errormessage\n";
		print "&nbsp;</class>\n";
		&Header::closebox();
	}
}
sub setup_default_networks
{
	my %netsettings=();
	my $defaultNetworks = shift;
	#my $netsettings = shift;
	&General::readhash("/var/ipfire/ethernet/settings", \%netsettings);
	
	# Get current defined networks (Red, Green, Blue, Orange)
	$defaultNetworks->{$Lang::tr{'fwhost any'}}{'IPT'} = "0.0.0.0/0.0.0.0";
	
		
	$defaultNetworks->{$Lang::tr{'fwhost green'}}{'IPT'} = "$netsettings{'GREEN_NETADDRESS'}/$netsettings{'GREEN_NETMASK'}";
	

	if ($netsettings{'ORANGE_DEV'} ne ''){
		$defaultNetworks->{$Lang::tr{'fwhost orange'}}{'IPT'} = "$netsettings{'ORANGE_NETADDRESS'}/$netsettings{'ORANGE_NETMASK'}";
	}

	if ($netsettings{'BLUE_DEV'} ne ''){
		$defaultNetworks->{$Lang::tr{'fwhost blue'}}{'IPT'} = "$netsettings{'BLUE_NETADDRESS'}/$netsettings{'BLUE_NETMASK'}";
	}

	# OpenVPN
	if(-e "${General::swroot}/ovpn/settings")
	{
		my %ovpnSettings = ();
		&General::readhash("${General::swroot}/ovpn/settings", \%ovpnSettings);

		# OpenVPN on Red?
		if(defined($ovpnSettings{'DOVPN_SUBNET'}))
		{
			my ($ip,$sub) = split(/\//,$ovpnSettings{'DOVPN_SUBNET'});
			$sub=&General::iporsubtocidr($sub);
			my @tempovpnsubnet = split("\/", $ovpnSettings{'DOVPN_SUBNET'});
			$defaultNetworks->{'OpenVPN ' .$ip."/".$sub}{'ADR'} = $tempovpnsubnet[0];
		}
	} # end OpenVPN

	open(FILE, "${General::swroot}/ethernet/aliases") or die 'Unable to open aliases file.';
	my @current = <FILE>;
	close(FILE);
	my $ctr = 0;
	foreach my $line (@current)
	{
		if ($line ne ''){
			chomp($line);
			my @temp = split(/\,/,$line);
			if ($temp[2] eq '') {
				$temp[2] = "Alias $ctr : $temp[0]";
			}
			$defaultNetworks->{$temp[2]}{'IPT'} = "$temp[0]";
			$ctr++;
		}
	}
}
sub deletefromgrp
{
	my $target=shift;
	my $config=shift;
	my %hash=();
	&General::readhasharray("$config",\%hash);
	foreach my $key (keys %hash) {
		if($hash{$key}[2] eq $target){
			delete $hash{$key};
		}
	}
	&General::writehasharray("$config",\%hash);
	
}
sub updategrp
{
	my %hash=();
	&General::readhasharray("$configgrp",\%hash);
	foreach my $key (keys %hash) {
		if($hash{$key}[2] eq $fwhostsettings{'OLDNAME'}){
			$hash{$key}[2] = $fwhostsettings{'HOSTNAME'};
			$hash{$key}[3] = $fwhostsettings{'IP'};
		}
	}
	&General::writehasharray("$configgrp",\%hash);
	
}
sub plausicheck
{
	
	my $edit=shift;
	#check hostname
	if (!&General::validhostname($fwhostsettings{'HOSTNAME'}))
	{
		$errormessage=$errormessage.$Lang::tr{'fwhost err name'};
		$fwhostsettings{'BLK_IP'}='readonly';
		$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
		if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
	}
	#check if name collides with CCD Netname
	if ($fwhostsettings{'NOCHECK'} eq ''){
		&General::readhasharray("$configccdnet", \%ccdnet);
		foreach my $key (keys %ccdnet) {
			if($ccdnet{$key}[0] eq $fwhostsettings{'HOSTNAME'}){
				$errormessage=$errormessage.$Lang::tr{'fwhost err isccdnet'};;
				$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
				last;
			}
		}
	}
	#check if IP collides with CCD NetIP
	if ($fwhostsettings{'NOCHECK'} eq ''){
		&General::readhasharray("$configccdnet", \%ccdnet);
		foreach my $key (keys %ccdnet) {
			my $test=(&General::getnetworkip($fwhostsettings{'IP'},&General::iporsubtocidr($fwhostsettings{'SUBNET'})))."/".$fwhostsettings{'SUBNET'};
			if($ccdnet{$key}[1] eq $test){
				$errormessage=$errormessage.$Lang::tr{'fwhost err isccdipnet'};
				$fwhostsettings{'IP'} = $fwhostsettings{'orgip'};
				$fwhostsettings{'SUBNET'} = $fwhostsettings{'orgsubnet'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
				last;
			}
		}
	}
	
	
	#check if name collides with CCD Hostname
	&General::readhasharray("$configccdhost", \%ccdhost);
	foreach my $key (keys %ccdhost) {
		my ($ip,$sub)=split(/\//,$ccdhost{$key}[33]);
		if($ip eq $fwhostsettings{'IP'}){
			$errormessage=$Lang::tr{'fwhost err isccdiphost'};
			if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
			last;
		}
	}
	#check if IP collides with CCD HostIP (only hosts)
	if ($edit eq 'edithost')
	{
		foreach my $key (keys %ccdhost) {
			if($ccdhost{$key}[1] eq $fwhostsettings{'HOSTNAME'}){
				$errormessage=$Lang::tr{'fwhost err isccdhost'};
				$fwhostsettings{'IP'} = $fwhostsettings{'orgname'};
				if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
				last;
			}
		}
	}
	#check if network with this name already exists
	&General::readhasharray("$confignet", \%customnetwork);
	if (!&checkhash(\%customnetwork))
	{
		$errormessage=$errormessage."<br>".$Lang::tr{'fwhost err netexist'};
		$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
		if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
	}	
	#check if network ip already exists		
	if (!&checkip(\%customnetwork,1))
	{
		$errormessage=$errormessage."<br>".$Lang::tr{'fwhost err net'};
		if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
	}	
	#check if host with this name already exists
	&General::readhasharray("$confighost", \%customhost);
	if (!&checkhash(\%customhost))
	{
		$errormessage=$errormessage."<br>".$Lang::tr{'fwhost err hostexist'};
		$fwhostsettings{'HOSTNAME'} = $fwhostsettings{'orgname'};
		if ($fwhostsettings{'update'} eq 'on'){$fwhostsettings{'ACTION'}=$edit;}
	}
	#check if host with this ip already exists
	if (!&checkip(\%customhost,2))
	{
		$errormessage=$errormessage."<br>".$Lang::tr{'fwhost err ipcheck'};
		
	}
		
	
	return;
}
sub rules
{
	$errormessage=$errormessage." Reread rules here...";
}
sub decrease
{
	my $grp=$_[0];
	#&General::readhasharray("$configgrp", \%customgrp);
	&General::readhasharray("$confignet", \%customnetwork);
	&General::readhasharray("$confighost", \%customhost);
	foreach my $key (sort keys %customgrp ){
		if ( ($customgrp{$key}[0] eq $grp) && ($customgrp{$key}[4] eq 'Custom Network')){
			foreach my $key1 (sort keys %customnetwork){
				if ($customnetwork{$key1}[0] eq $customgrp{$key}[2]){
					$customnetwork{$key1}[3]=$customnetwork{$key1}[3]-1;
					last;
				}
			}
		}
		
		if (($customgrp{$key}[0] eq $grp) && ($customgrp{$key}[4] eq 'Custom Host')){
			foreach my $key2 (sort keys %customhost){
				if ($customhost{$key2}[0] eq $customgrp{$key}[2]){
					$customhost{$key2}[3]=$customhost{$key2}[3]-1;
					last;
				}
			}
				
		}
	}
	&General::writehasharray("$confignet", \%customnetwork);
	&General::writehasharray("$confighost", \%customhost);
}
&Header::closebigbox();
&Header::closepage();
